"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __makeTemplateObject = (this && this.__makeTemplateObject) || function (cooked, raw) {
    if (Object.defineProperty) { Object.defineProperty(cooked, "raw", { value: raw }); } else { cooked.raw = raw; }
    return cooked;
};
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_transformer_core_1 = require("graphql-transformer-core");
var resources_1 = require("./resources");
var graphql_1 = require("graphql");
var graphql_transformer_common_1 = require("graphql-transformer-common");
var graphql_mapping_template_1 = require("graphql-mapping-template");
var graphql_2 = require("graphql");
var validateAuthMode = function (mode) {
    if (mode !== 'API_KEY' &&
        mode !== 'AMAZON_COGNITO_USER_POOLS') {
        throw new Error("Invalid auth mode " + mode);
    }
};
var ModelAuthTransformer = /** @class */ (function (_super) {
    __extends(ModelAuthTransformer, _super);
    function ModelAuthTransformer(config) {
        var _this = _super.call(this, 'ModelAuthTransformer', graphql_transformer_core_1.gql(templateObject_1 || (templateObject_1 = __makeTemplateObject(["\n            directive @auth(rules: [AuthRule!]!) on OBJECT | FIELD_DEFINITION\n            input AuthRule {\n                # Specifies the auth rule's strategy. Allowed values are 'owner' and 'groups'.\n                allow: AuthStrategy!\n\n                # Specifies the name of the claim to look for on the request's JWT token\n                # from Cognito User Pools (and in the future OIDC) that contains the identity\n                # of the user. If 'allow' is 'groups', this value should point to a list of groups\n                # in the claims. If 'allow' is 'owner', this value should point to the logged in user identity string.\n                # Defaults to \"cognito:username\" for Cognito User Pools auth.\n                identityField: String\n\n                # Allowed when the 'allow' argument is 'owner'.\n                # Specifies the field of type String or [String] that contains owner(s) that can access the object.\n                ownerField: String # defaults to \"owner\"\n\n                # Allowed when the 'allow' argument is 'groups'.\n                # Specifies the field of type String or [String] that contains group(s) that can access the object.\n                groupsField: String\n\n                # Allowed when the 'allow' argument is 'groups'.\n                # Specifies a static list of groups that should have access to the object.\n                groups: [String]\n\n                # Specifies operations to which this auth rule should be applied.\n                operations: [ModelOperation]\n\n                # Deprecated. It is recommended to use the 'operations' arguments.\n                queries: [ModelQuery]\n                    @deprecated(reason: \"The 'queries' argument will be replaced by the 'operations' argument in a future release.\")\n\n                # Deprecated. It is recommended to use the 'operations' arguments.\n                mutations: [ModelMutation]\n                    @deprecated(reason: \"The 'mutations' argument will be replaced by the 'operations' argument in a future release.\")\n            }\n            enum AuthStrategy { owner groups }\n            enum ModelOperation { create update delete read }\n            enum ModelQuery\n                @deprecated(reason: \"ModelQuery will be replaced by the 'ModelOperation' in a future release.\")\n            {\n                get\n                list\n            }\n            enum ModelMutation\n                @deprecated(reason: \"ModelMutation will be replaced by the 'ModelOperation' in a future release.\")\n            {\n                create\n                update\n                delete\n            }\n            "], ["\n            directive @auth(rules: [AuthRule!]!) on OBJECT | FIELD_DEFINITION\n            input AuthRule {\n                # Specifies the auth rule's strategy. Allowed values are 'owner' and 'groups'.\n                allow: AuthStrategy!\n\n                # Specifies the name of the claim to look for on the request's JWT token\n                # from Cognito User Pools (and in the future OIDC) that contains the identity\n                # of the user. If 'allow' is 'groups', this value should point to a list of groups\n                # in the claims. If 'allow' is 'owner', this value should point to the logged in user identity string.\n                # Defaults to \"cognito:username\" for Cognito User Pools auth.\n                identityField: String\n\n                # Allowed when the 'allow' argument is 'owner'.\n                # Specifies the field of type String or [String] that contains owner(s) that can access the object.\n                ownerField: String # defaults to \"owner\"\n\n                # Allowed when the 'allow' argument is 'groups'.\n                # Specifies the field of type String or [String] that contains group(s) that can access the object.\n                groupsField: String\n\n                # Allowed when the 'allow' argument is 'groups'.\n                # Specifies a static list of groups that should have access to the object.\n                groups: [String]\n\n                # Specifies operations to which this auth rule should be applied.\n                operations: [ModelOperation]\n\n                # Deprecated. It is recommended to use the 'operations' arguments.\n                queries: [ModelQuery]\n                    @deprecated(reason: \"The 'queries' argument will be replaced by the 'operations' argument in a future release.\")\n\n                # Deprecated. It is recommended to use the 'operations' arguments.\n                mutations: [ModelMutation]\n                    @deprecated(reason: \"The 'mutations' argument will be replaced by the 'operations' argument in a future release.\")\n            }\n            enum AuthStrategy { owner groups }\n            enum ModelOperation { create update delete read }\n            enum ModelQuery\n                @deprecated(reason: \"ModelQuery will be replaced by the 'ModelOperation' in a future release.\")\n            {\n                get\n                list\n            }\n            enum ModelMutation\n                @deprecated(reason: \"ModelMutation will be replaced by the 'ModelOperation' in a future release.\")\n            {\n                create\n                update\n                delete\n            }\n            "])))) || this;
        /**
         * Updates the GraphQL API record to use user pool auth.
         */
        _this.updateAPIForUserPools = function (ctx) {
            var apiRecord = ctx.getResource(graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID);
            var updated = _this.resources.updateGraphQLAPIWithAuth(apiRecord, _this.config.authMode);
            ctx.setResource(graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID, updated);
        };
        _this.before = function (ctx) {
            var template = _this.resources.initTemplate();
            ctx.mergeResources(template.Resources);
            ctx.mergeParameters(template.Parameters);
            ctx.mergeOutputs(template.Outputs);
            ctx.mergeConditions(template.Conditions);
            _this.updateAPIForUserPools(ctx);
        };
        /**
         * Implement the transform for an object type. Depending on which operations are to be protected
         */
        _this.object = function (def, directive, ctx) {
            var get = function (s) { return function (arg) { return arg.name.value === s; }; };
            var getArg = function (arg, dflt) {
                var argument = directive.arguments.find(get(arg));
                return argument ? graphql_2.valueFromASTUntyped(argument.value) : dflt;
            };
            var modelDirective = def.directives.find(function (dir) { return dir.name.value === 'model'; });
            if (!modelDirective) {
                throw new graphql_transformer_core_1.InvalidDirectiveError('Types annotated with @auth must also be annotated with @model.');
            }
            // Get and validate the auth rules.
            var rules = getArg('rules', []);
            _this.validateRules(rules);
            var _a = _this.splitRules(rules), operationRules = _a.operationRules, queryRules = _a.queryRules;
            // For each operation evaluate the rules and apply the changes to the relevant resolver.
            _this.protectCreateMutation(ctx, graphql_transformer_common_1.ResolverResourceIDs.DynamoDBCreateResolverResourceID(def.name.value), operationRules.create, def);
            _this.protectUpdateMutation(ctx, graphql_transformer_common_1.ResolverResourceIDs.DynamoDBUpdateResolverResourceID(def.name.value), operationRules.update, def);
            _this.protectDeleteMutation(ctx, graphql_transformer_common_1.ResolverResourceIDs.DynamoDBDeleteResolverResourceID(def.name.value), operationRules.delete, def);
            _this.protectGetQuery(ctx, graphql_transformer_common_1.ResolverResourceIDs.DynamoDBGetResolverResourceID(def.name.value), queryRules.get);
            _this.protectListQuery(ctx, graphql_transformer_common_1.ResolverResourceIDs.DynamoDBListResolverResourceID(def.name.value), queryRules.list);
            _this.protectConnections(ctx, def, operationRules.read);
            _this.protectQueries(ctx, def, operationRules.read);
        };
        _this.field = function (parent, definition, directive, ctx) {
            if (parent.kind === graphql_1.Kind.INTERFACE_TYPE_DEFINITION) {
                throw new graphql_transformer_core_1.InvalidDirectiveError("The @auth directive cannot be placed on an interface's field. See " + parent.name.value + definition.name.value);
            }
            var get = function (s) { return function (arg) { return arg.name.value === s; }; };
            var getArg = function (arg, dflt) {
                var argument = directive.arguments.find(get(arg));
                return argument ? graphql_2.valueFromASTUntyped(argument.value) : dflt;
            };
            if (parent.name.value === ctx.getQueryTypeName() ||
                parent.name.value === ctx.getMutationTypeName() ||
                parent.name.value === ctx.getSubscriptionTypeName()) {
                console.warn("Be careful when using @auth directives on a field in a root type. @auth directives on field definitions use the source object to perform authorization logic and the source will be an empty object for fields on root types. Static group authorization should perform as expected.");
            }
            // Get and validate the auth rules.
            var rules = getArg('rules', []);
            _this.validateFieldRules(rules);
            var isOpRule = function (op) { return function (rule) {
                if (rule.operations) {
                    var matchesOp = rule.operations.find(function (o) { return o === op; });
                    return Boolean(matchesOp);
                }
                else if (rule.operations === null) {
                    return false;
                }
                return true;
            }; };
            var isReadRule = isOpRule('read');
            var isCreateRule = isOpRule('create');
            var isUpdateRule = isOpRule('update');
            var isDeleteRule = isOpRule('delete');
            // The field handler adds the read rule on the object
            var readRules = rules.filter(function (rule) { return isReadRule(rule); });
            _this.protectField(ctx, parent.name.value, definition.name.value, readRules);
            // Protect mutations when objects including this field are trying to be created.
            var createRules = rules.filter(function (rule) { return isCreateRule(rule); });
            _this.protectCreateForField(ctx, parent, definition, createRules);
            // Protect update mutations when objects inluding this field are trying to be updated.
            var updateRules = rules.filter(function (rule) { return isUpdateRule(rule); });
            _this.protectUpdateForField(ctx, parent, definition, updateRules);
            // Delete operations are only protected by @auth directives on objects.
            var deleteRules = rules.filter(function (rule) { return isDeleteRule(rule); });
            _this.protectDeleteForField(ctx, parent, definition, deleteRules);
        };
        _this.config = config || { authMode: 'API_KEY' };
        validateAuthMode(_this.config.authMode);
        _this.resources = new resources_1.ResourceFactory();
        return _this;
    }
    ModelAuthTransformer.prototype.protectField = function (ctx, typeName, fieldName, rules) {
        if (rules && rules.length) {
            var resolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.ResolverResourceID(typeName, fieldName);
            // If the resolver exists (e.g. @connection use it else make a blank one against None)
            var resolver = ctx.getResource(resolverResourceId);
            if (!resolver) {
                // If we need a none data source for the blank resolver, add it.
                var noneDS = ctx.getResource(graphql_transformer_common_1.ResourceConstants.RESOURCES.NoneDataSource);
                if (!noneDS) {
                    ctx.setResource(graphql_transformer_common_1.ResourceConstants.RESOURCES.NoneDataSource, this.resources.noneDataSource());
                }
                // We also need to add a stack mapping so that this resolver is added to the model stack.
                ctx.addToStackMapping("" + typeName, resolverResourceId);
                resolver = this.resources.blankResolver(typeName, fieldName);
            }
            var authExpression = this.authorizationExpressionOnSingleObject(rules, 'ctx.source');
            // If a resolver exists, a @connection for example. Prepend it to the req.
            var templateParts = [
                graphql_mapping_template_1.print(authExpression),
                resolver.Properties.RequestMappingTemplate
            ];
            resolver.Properties.RequestMappingTemplate = templateParts.join('\n\n');
            ctx.setResource(resolverResourceId, resolver);
        }
    };
    ModelAuthTransformer.prototype.protectUpdateForField = function (ctx, parent, field, rules) {
        var resolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBUpdateResolverResourceID(parent.name.value);
        this.protectUpdateMutation(ctx, resolverResourceId, rules, parent, field);
    };
    ModelAuthTransformer.prototype.protectDeleteForField = function (ctx, parent, field, rules) {
        var resolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBUpdateResolverResourceID(parent.name.value);
        this.protectDeleteMutation(ctx, resolverResourceId, rules, parent, field);
    };
    /**
     * Protects a create mutation based on an @auth rule specified on a @model field.
     * @param ctx The context.
     * @param typeName The parent type name.
     * @param fieldName The name of the field with the @auth directive.
     * @param rules The set of rules that should be applied to create operations.
     */
    ModelAuthTransformer.prototype.protectCreateForField = function (ctx, parent, field, rules) {
        var typeName = parent.name.value;
        var resolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.DynamoDBCreateResolverResourceID(typeName);
        var createResolverResource = ctx.getResource(resolverResourceId);
        if (rules && rules.length && createResolverResource) {
            // Break the rules out by strategy.
            var staticGroupAuthorizationRules = this.getStaticGroupRules(rules);
            var dynamicGroupAuthorizationRules = this.getDynamicGroupRules(rules);
            var ownerAuthorizationRules = this.getOwnerRules(rules);
            // Generate the expressions to validate each strategy.
            var staticGroupAuthorizationExpression = this.resources.staticGroupAuthorizationExpression(staticGroupAuthorizationRules);
            // In create mutations, the dynamic group and ownership authorization checks
            // are done before calling PutItem.
            var dynamicGroupAuthorizationExpression = this.resources.dynamicGroupAuthorizationExpressionForCreateOperationsByField(dynamicGroupAuthorizationRules, field.name.value);
            var fieldIsList = function (fieldName) {
                var field = parent.fields.find(function (field) { return field.name.value === fieldName; });
                if (field) {
                    return graphql_transformer_common_1.isListType(field.type);
                }
                return false;
            };
            var ownerAuthorizationExpression = this.resources.ownerAuthorizationExpressionForCreateOperationsByField(ownerAuthorizationRules, field.name.value, fieldIsList);
            var throwIfUnauthorizedExpression = this.resources.throwIfUnauthorized();
            var templateParts = [
                graphql_mapping_template_1.print(graphql_mapping_template_1.iff(graphql_mapping_template_1.raw("$ctx.args.input.containsKey(\"" + field.name.value + "\")"), graphql_mapping_template_1.compoundExpression([
                    staticGroupAuthorizationExpression,
                    graphql_mapping_template_1.newline(),
                    dynamicGroupAuthorizationExpression,
                    graphql_mapping_template_1.newline(),
                    ownerAuthorizationExpression,
                    graphql_mapping_template_1.newline(),
                    throwIfUnauthorizedExpression
                ]))),
                createResolverResource.Properties.RequestMappingTemplate
            ];
            createResolverResource.Properties.RequestMappingTemplate = templateParts.join('\n\n');
            ctx.setResource(resolverResourceId, createResolverResource);
        }
    };
    /**
     * Takes a flat list of rules, each containing their own list of operations (or queries/mutations if an old API).
     * This method splits those rules into buckets keyed by operation and implements some logic for backwards compatibility.
     * @param rules The list of auth rules
     */
    ModelAuthTransformer.prototype.splitRules = function (rules) {
        // Create a reverse index on rules from operation -> rules list.
        var queryRules = {
            get: [],
            list: [],
        };
        var operationRules = {
            create: [],
            update: [],
            delete: [],
            read: []
        };
        var matchQuery = function (op) { return function (rule) {
            if (rule.queries) {
                var matchesOp = rule.queries.find(function (o) { return o === op; });
                return Boolean(matchesOp);
            }
            else if (rule.queries === null) {
                return false;
            }
            return true;
        }; };
        var matchMutation = function (op) { return function (rule) {
            if (rule.mutations) {
                var matchesOp = rule.mutations.find(function (o) { return o === op; });
                return Boolean(matchesOp);
            }
            else if (rule.mutations === null) {
                return false;
            }
            return true;
        }; };
        var matchOperation = function (op) { return function (rule) {
            if (rule.operations) {
                var matchesOp = rule.operations.find(function (o) { return o === op; });
                return Boolean(matchesOp);
            }
            else if (rule.operations === null) {
                return false;
            }
            return true;
        }; };
        for (var _i = 0, rules_1 = rules; _i < rules_1.length; _i++) {
            var rule = rules_1[_i];
            // If operations is provided, then it takes precendence.
            if (isTruthyOrNull(rule.operations)) {
                // If operations is given use it.
                if (matchOperation('read')(rule)) {
                    queryRules.get.push(rule);
                    queryRules.list.push(rule);
                    operationRules.read.push(rule);
                }
                if (matchOperation('create')(rule)) {
                    operationRules.create.push(rule);
                }
                if (matchOperation('update')(rule)) {
                    operationRules.update.push(rule);
                }
                if (matchOperation('delete')(rule)) {
                    operationRules.delete.push(rule);
                }
            }
            else {
                // If operations is not provided, either use the default behavior or deprecated
                // behavior from the queries/mutations arguments for backwards compatibility.
                // Handle default or deprecated query use case
                if (isUndefined(rule.queries)) {
                    // If both operations and queries are undefined, default to read operation protection.
                    // This is the default behavior. E.G. @auth(rules: [{ allow: owner }])
                    queryRules.get.push(rule);
                    queryRules.list.push(rule);
                    operationRules.read.push(rule);
                }
                else {
                    // If operations is undefined & queries is defined, use queries.
                    // This is the old behavior for backwards compatibility.
                    if (matchQuery('get')(rule)) {
                        queryRules.get.push(rule);
                    }
                    if (matchQuery('list')(rule)) {
                        queryRules.list.push(rule);
                    }
                }
                // Handle default or deprecated mutation use case
                if (isUndefined(rule.mutations)) {
                    // If both operations and mutations are undefined, default to create, update, delete
                    // operation protection. This is the default behavior. E.G. @auth(rules: [{ allow: owner }])
                    operationRules.create.push(rule);
                    operationRules.update.push(rule);
                    operationRules.delete.push(rule);
                }
                else {
                    // If operations is undefined & mutations is defined, use mutations.
                    // This is the old behavior for backwards compatibility.
                    if (matchMutation('create')(rule)) {
                        operationRules.create.push(rule);
                    }
                    if (matchMutation('update')(rule)) {
                        operationRules.update.push(rule);
                    }
                    if (matchMutation('delete')(rule)) {
                        operationRules.delete.push(rule);
                    }
                }
            }
        }
        return {
            operationRules: operationRules,
            queryRules: queryRules
        };
    };
    ModelAuthTransformer.prototype.validateRules = function (rules) {
        for (var _i = 0, rules_2 = rules; _i < rules_2.length; _i++) {
            var rule = rules_2[_i];
            var queries = rule.queries, mutations = rule.mutations, operations = rule.operations;
            if (mutations && operations) {
                console.warn("It is not recommended to use 'mutations' and 'operations'. The 'operations' argument will be used.");
            }
            if (queries && operations) {
                console.warn("It is not recommended to use 'queries' and 'operations'. The 'operations' argument will be used.");
            }
        }
    };
    ModelAuthTransformer.prototype.validateFieldRules = function (rules) {
        for (var _i = 0, rules_3 = rules; _i < rules_3.length; _i++) {
            var rule = rules_3[_i];
            var queries = rule.queries, mutations = rule.mutations;
            if (queries || mutations) {
                throw new graphql_transformer_core_1.InvalidDirectiveError("@auth directives used on field definitions may not specify the 'queries' or 'mutations' arguments. All @auth directives used on field definitions are performed when the field is resolved and can be thought of as 'read' operations.");
            }
        }
    };
    /**
     * Protect get queries.
     * If static group:
     *  If statically authorized then allow the operation. Stop.
     * If owner and/or dynamic group:
     *  If the result item satisfies the owner/group authorization condition
     *  then allow it.
     * @param ctx The transformer context.
     * @param resolverResourceId The logical id of the get resolver.
     * @param rules The auth rules to apply.
     */
    ModelAuthTransformer.prototype.protectGetQuery = function (ctx, resolverResourceId, rules) {
        var resolver = ctx.getResource(resolverResourceId);
        if (!rules || rules.length === 0 || !resolver) {
            return;
        }
        else {
            var authExpression = this.authorizationExpressionOnSingleObject(rules);
            var templateParts = [
                graphql_mapping_template_1.print(authExpression),
                resolver.Properties.ResponseMappingTemplate
            ];
            resolver.Properties.ResponseMappingTemplate = templateParts.join('\n\n');
            ctx.setResource(resolverResourceId, resolver);
        }
    };
    ModelAuthTransformer.prototype.authorizationExpressionOnSingleObject = function (rules, objectPath) {
        if (objectPath === void 0) { objectPath = 'ctx.result'; }
        // Break the rules out by strategy.
        var staticGroupAuthorizationRules = this.getStaticGroupRules(rules);
        var dynamicGroupAuthorizationRules = this.getDynamicGroupRules(rules);
        var ownerAuthorizationRules = this.getOwnerRules(rules);
        // Generate the expressions to validate each strategy.
        var staticGroupAuthorizationExpression = this.resources.staticGroupAuthorizationExpression(staticGroupAuthorizationRules);
        var dynamicGroupAuthorizationExpression = this.resources.dynamicGroupAuthorizationExpressionForReadOperations(dynamicGroupAuthorizationRules, objectPath);
        var ownerAuthorizationExpression = this.resources.ownerAuthorizationExpressionForReadOperations(ownerAuthorizationRules, objectPath);
        var throwIfUnauthorizedExpression = this.resources.throwIfUnauthorized();
        // Update the existing resolver with the authorization checks.
        var templateExpression = graphql_mapping_template_1.compoundExpression([
            staticGroupAuthorizationExpression,
            graphql_mapping_template_1.newline(),
            dynamicGroupAuthorizationExpression,
            graphql_mapping_template_1.newline(),
            ownerAuthorizationExpression,
            graphql_mapping_template_1.newline(),
            throwIfUnauthorizedExpression
        ]);
        return templateExpression;
    };
    /**
     * Protect list queries.
     * If static group:
     *  If the user is statically authorized then return items and stop.
     * If dynamic group and/or owner:
     *  Loop through all items and find items that satisfy any of the group or
     *  owner conditions.
     * @param ctx The transformer context.
     * @param resolverResourceId The logical id of the resolver to be updated in the CF template.
     * @param rules The set of rules that apply to the operation.
     */
    ModelAuthTransformer.prototype.protectListQuery = function (ctx, resolverResourceId, rules) {
        var resolver = ctx.getResource(resolverResourceId);
        if (!rules || rules.length === 0 || !resolver) {
            return;
        }
        else {
            var authExpression = this.authorizationExpressionForListResult(rules);
            var templateParts = [
                graphql_mapping_template_1.print(authExpression),
                resolver.Properties.ResponseMappingTemplate
            ];
            resolver.Properties.ResponseMappingTemplate = templateParts.join('\n\n');
            ctx.setResource(resolverResourceId, resolver);
        }
    };
    /**
     * Returns a VTL expression that will authorize a list of results based on a set of auth rules.
     * @param rules The auth rules.
     */
    ModelAuthTransformer.prototype.authorizationExpressionForListResult = function (rules) {
        // Break the rules out by strategy.
        var staticGroupAuthorizationRules = this.getStaticGroupRules(rules);
        var dynamicGroupAuthorizationRules = this.getDynamicGroupRules(rules);
        var ownerAuthorizationRules = this.getOwnerRules(rules);
        // Generate the expressions to validate each strategy.
        var staticGroupAuthorizationExpression = this.resources.staticGroupAuthorizationExpression(staticGroupAuthorizationRules);
        // In list queries, the dynamic group and ownership authorization checks
        // occur on a per item basis. The helpers take the variable names
        // as parameters to allow for this use case.
        var dynamicGroupAuthorizationExpression = this.resources.dynamicGroupAuthorizationExpressionForReadOperations(dynamicGroupAuthorizationRules, 'item', graphql_transformer_common_1.ResourceConstants.SNIPPETS.IsLocalDynamicGroupAuthorizedVariable, graphql_mapping_template_1.raw("false"));
        var ownerAuthorizationExpression = this.resources.ownerAuthorizationExpressionForReadOperations(ownerAuthorizationRules, 'item', graphql_transformer_common_1.ResourceConstants.SNIPPETS.IsLocalOwnerAuthorizedVariable, graphql_mapping_template_1.raw("false"));
        var appendIfLocallyAuthorized = this.resources.appendItemIfLocallyAuthorized();
        var ifNotStaticallyAuthedFilterObjects = graphql_mapping_template_1.iff(graphql_mapping_template_1.raw("! $" + graphql_transformer_common_1.ResourceConstants.SNIPPETS.IsStaticGroupAuthorizedVariable), graphql_mapping_template_1.compoundExpression([
            graphql_mapping_template_1.set(graphql_mapping_template_1.ref('items'), graphql_mapping_template_1.list([])),
            graphql_mapping_template_1.forEach(graphql_mapping_template_1.ref('item'), graphql_mapping_template_1.ref('ctx.result.items'), [
                dynamicGroupAuthorizationExpression,
                graphql_mapping_template_1.newline(),
                ownerAuthorizationExpression,
                graphql_mapping_template_1.newline(),
                appendIfLocallyAuthorized
            ]),
            graphql_mapping_template_1.set(graphql_mapping_template_1.ref('ctx.result.items'), graphql_mapping_template_1.ref('items'))
        ]));
        return graphql_mapping_template_1.compoundExpression([
            staticGroupAuthorizationExpression,
            graphql_mapping_template_1.newline(),
            graphql_mapping_template_1.comment('[Start] If not static group authorized, filter items'),
            ifNotStaticallyAuthedFilterObjects,
            graphql_mapping_template_1.comment('[End] If not static group authorized, filter items')
        ]);
    };
    /**
     * Inject auth rules for create mutations.
     * If owner auth:
     *  If the owner field exists in the input, validate that it against the identity.
     *  If the owner field dne in the input, insert the identity.
     * If group:
     *  If the user is static group authorized allow operation no matter what.
     *  If dynamic group and the input defines a group(s) validate it against the identity.
     * @param ctx
     * @param resolverResourceId
     * @param rules
     */
    ModelAuthTransformer.prototype.protectCreateMutation = function (ctx, resolverResourceId, rules, parent) {
        var resolver = ctx.getResource(resolverResourceId);
        if (!rules || rules.length === 0 || !resolver) {
            return;
        }
        else {
            // Break the rules out by strategy.
            var staticGroupAuthorizationRules = this.getStaticGroupRules(rules);
            var dynamicGroupAuthorizationRules = this.getDynamicGroupRules(rules);
            var ownerAuthorizationRules = this.getOwnerRules(rules);
            // Generate the expressions to validate each strategy.
            var staticGroupAuthorizationExpression = this.resources.staticGroupAuthorizationExpression(staticGroupAuthorizationRules);
            // In create mutations, the dynamic group and ownership authorization checks
            // are done before calling PutItem.
            var dynamicGroupAuthorizationExpression = this.resources.dynamicGroupAuthorizationExpressionForCreateOperations(dynamicGroupAuthorizationRules);
            var fieldIsList = function (fieldName) {
                var field = parent.fields.find(function (field) { return field.name.value === fieldName; });
                if (field) {
                    return graphql_transformer_common_1.isListType(field.type);
                }
                return false;
            };
            var ownerAuthorizationExpression = this.resources.ownerAuthorizationExpressionForCreateOperations(ownerAuthorizationRules, fieldIsList);
            var throwIfUnauthorizedExpression = this.resources.throwIfUnauthorized();
            var templateParts = [
                graphql_mapping_template_1.print(graphql_mapping_template_1.compoundExpression([
                    staticGroupAuthorizationExpression,
                    graphql_mapping_template_1.newline(),
                    dynamicGroupAuthorizationExpression,
                    graphql_mapping_template_1.newline(),
                    ownerAuthorizationExpression,
                    graphql_mapping_template_1.newline(),
                    throwIfUnauthorizedExpression
                ])),
                resolver.Properties.RequestMappingTemplate
            ];
            resolver.Properties.RequestMappingTemplate = templateParts.join('\n\n');
            ctx.setResource(resolverResourceId, resolver);
        }
    };
    /**
     * Protect update and delete mutations.
     * If Owner:
     *  Update the conditional expression such that the update only works if
     *  the user is the owner.
     * If dynamic group:
     *  Update the conditional expression such that it succeeds if the user is
     *  dynamic group authorized. If the operation is also owner authorized this
     *  should be joined with an OR expression.
     * If static group:
     *  If the user is statically authorized then allow no matter what. This can
     *  be done by removing the conditional expression as long as static group
     *  auth is always checked last.
     * @param ctx The transformer context.
     * @param resolverResourceId The logical id of the resolver in the template.
     * @param rules The list of rules to apply.
     */
    ModelAuthTransformer.prototype.protectUpdateOrDeleteMutation = function (ctx, resolverResourceId, rules, parent, field, ifCondition) {
        var resolver = ctx.getResource(resolverResourceId);
        if (!rules || rules.length === 0 || !resolver) {
            return;
        }
        else {
            // Break the rules out by strategy.
            var staticGroupAuthorizationRules = this.getStaticGroupRules(rules);
            var dynamicGroupAuthorizationRules = this.getDynamicGroupRules(rules);
            var ownerAuthorizationRules = this.getOwnerRules(rules);
            // Generate the expressions to validate each strategy.
            var staticGroupAuthorizationExpression = this.resources.staticGroupAuthorizationExpression(staticGroupAuthorizationRules);
            // In create mutations, the dynamic group and ownership authorization checks
            // are done before calling PutItem.
            var dynamicGroupAuthorizationExpression = this.resources.dynamicGroupAuthorizationExpressionForUpdateOrDeleteOperations(dynamicGroupAuthorizationRules, field ? field.name.value : undefined);
            var fieldIsList = function (fieldName) {
                var field = parent.fields.find(function (field) { return field.name.value === fieldName; });
                if (field) {
                    return graphql_transformer_common_1.isListType(field.type);
                }
                return false;
            };
            var ownerAuthorizationExpression = this.resources.ownerAuthorizationExpressionForUpdateOrDeleteOperations(ownerAuthorizationRules, fieldIsList, field ? field.name.value : undefined);
            var collectAuthCondition = this.resources.collectAuthCondition();
            var ifNotStaticallyAuthedCreateAuthCondition = graphql_mapping_template_1.iff(graphql_mapping_template_1.raw("! $" + graphql_transformer_common_1.ResourceConstants.SNIPPETS.IsStaticGroupAuthorizedVariable), graphql_mapping_template_1.compoundExpression([
                dynamicGroupAuthorizationExpression,
                graphql_mapping_template_1.newline(),
                ownerAuthorizationExpression,
                graphql_mapping_template_1.newline(),
                collectAuthCondition
            ]));
            var throwIfNotStaticGroupAuthorizedOrAuthConditionIsEmpty = this.resources.throwIfNotStaticGroupAuthorizedOrAuthConditionIsEmpty();
            var authorizationLogic = graphql_mapping_template_1.compoundExpression([
                staticGroupAuthorizationExpression,
                graphql_mapping_template_1.newline(),
                ifNotStaticallyAuthedCreateAuthCondition,
                graphql_mapping_template_1.newline(),
                throwIfNotStaticGroupAuthorizedOrAuthConditionIsEmpty
            ]);
            var templateParts = [
                graphql_mapping_template_1.print(field && ifCondition ?
                    graphql_mapping_template_1.iff(ifCondition, authorizationLogic) :
                    authorizationLogic),
                resolver.Properties.RequestMappingTemplate
            ];
            resolver.Properties.RequestMappingTemplate = templateParts.join('\n\n');
            ctx.setResource(resolverResourceId, resolver);
        }
    };
    /**
     * If we are protecting the mutation for a field level @auth directive, include
     * the necessary if condition.
     * @param ctx The transformer context
     * @param resolverResourceId The resolver resource id
     * @param rules The delete rules
     * @param parent The parent object
     * @param field The optional field
     */
    ModelAuthTransformer.prototype.protectUpdateMutation = function (ctx, resolverResourceId, rules, parent, field) {
        return this.protectUpdateOrDeleteMutation(ctx, resolverResourceId, rules, parent, field, field ? graphql_mapping_template_1.raw("$ctx.args.input.containsKey(\"" + field.name.value + "\")") : undefined);
    };
    /**
     * If we are protecting the mutation for a field level @auth directive, include
     * the necessary if condition.
     * @param ctx The transformer context
     * @param resolverResourceId The resolver resource id
     * @param rules The delete rules
     * @param parent The parent object
     * @param field The optional field
     */
    ModelAuthTransformer.prototype.protectDeleteMutation = function (ctx, resolverResourceId, rules, parent, field) {
        return this.protectUpdateOrDeleteMutation(ctx, resolverResourceId, rules, parent, field, field ? graphql_mapping_template_1.raw("$ctx.args.input.containsKey(\"" + field.name.value + "\") && $util.isNull($ctx.args.input.get(\"" + field.name.value + "\"))") : undefined);
    };
    /**
     * When read operations are protected via @auth, all @connection resolvers will be protected.
     * Find the directives & update their resolvers with auth logic
     */
    ModelAuthTransformer.prototype.protectConnections = function (ctx, def, rules) {
        var thisModelName = def.name.value;
        for (var _i = 0, _a = ctx.inputDocument.definitions; _i < _a.length; _i++) {
            var inputDef = _a[_i];
            if (inputDef.kind === graphql_1.Kind.OBJECT_TYPE_DEFINITION) {
                for (var _b = 0, _c = inputDef.fields; _b < _c.length; _b++) {
                    var field = _c[_b];
                    var returnTypeName = graphql_transformer_common_1.getBaseType(field.type);
                    if (fieldHasDirective(field, 'connection') && returnTypeName === thisModelName) {
                        var resolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.ResolverResourceID(inputDef.name.value, field.name.value);
                        if (graphql_transformer_common_1.isListType(field.type)) {
                            this.protectListQuery(ctx, resolverResourceId, rules);
                        }
                        else {
                            this.protectGetQuery(ctx, resolverResourceId, rules);
                        }
                    }
                }
            }
        }
    };
    /**
     * When read operations are protected via @auth, all secondary @key query resolvers will be protected.
     * Find the directives & update their resolvers with auth logic
     */
    ModelAuthTransformer.prototype.protectQueries = function (ctx, def, rules) {
        var secondaryKeyDirectivesWithQueries = (def.directives || []).filter(function (d) {
            var isKey = d.name.value === 'key';
            var args = graphql_transformer_core_1.getDirectiveArguments(d);
            // @key with a name is a secondary key.
            var isSecondaryKey = Boolean(args.name);
            var hasQueryField = Boolean(args.queryField);
            return isKey && isSecondaryKey && hasQueryField;
        });
        for (var _i = 0, secondaryKeyDirectivesWithQueries_1 = secondaryKeyDirectivesWithQueries; _i < secondaryKeyDirectivesWithQueries_1.length; _i++) {
            var keyWithQuery = secondaryKeyDirectivesWithQueries_1[_i];
            var args = graphql_transformer_core_1.getDirectiveArguments(keyWithQuery);
            var resolverResourceId = graphql_transformer_common_1.ResolverResourceIDs.ResolverResourceID(ctx.getQueryTypeName(), args.queryField);
            this.protectListQuery(ctx, resolverResourceId, rules);
        }
    };
    ModelAuthTransformer.prototype.getOwnerRules = function (rules) {
        return rules.filter(function (rule) { return rule.allow === 'owner'; });
    };
    ModelAuthTransformer.prototype.getStaticGroupRules = function (rules) {
        return rules.filter(function (rule) { return rule.allow === 'groups' && Boolean(rule.groups); });
    };
    ModelAuthTransformer.prototype.getDynamicGroupRules = function (rules) {
        return rules.filter(function (rule) { return rule.allow === 'groups' && !Boolean(rule.groups); });
    };
    return ModelAuthTransformer;
}(graphql_transformer_core_1.Transformer));
exports.ModelAuthTransformer = ModelAuthTransformer;
function fieldHasDirective(field, directiveName) {
    return field.directives && field.directives.length && Boolean(field.directives.find(function (d) { return d.name.value === directiveName; }));
}
function isTruthyOrNull(obj) {
    return obj || obj === null;
}
function isUndefined(obj) {
    return obj === undefined;
}
var templateObject_1;
//# sourceMappingURL=ModelAuthTransformer.js.map