"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var graphql_transformer_core_1 = require("graphql-transformer-core");
var graphql_transformer_common_1 = require("graphql-transformer-common");
var graphql_dynamodb_transformer_1 = require("graphql-dynamodb-transformer");
var ModelAuthTransformer_1 = require("../ModelAuthTransformer");
test('Test "read" auth operation', function () {
    var validSchema = "\n    type Post @model @auth(rules: [{allow: groups, groups: [\"Admin\", \"Dev\"], operations: [read]}]) {\n        id: ID!\n        title: String!\n        createdAt: String\n        updatedAt: String\n    }\n    ";
    var transformer = new graphql_transformer_core_1.default({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({ authMode: 'AMAZON_COGNITO_USER_POOLS' })
        ]
    });
    var out = transformer.transform(validSchema);
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Query.getPost.res.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Query.listPosts.res.vtl']).toContain('Authorization rule:');
});
test('Test "create", "update", "delete" auth operations', function () {
    var validSchema = "\n    type Post @model @auth(rules: [{allow: groups, groups: [\"Admin\", \"Dev\"], operations: [create, update, delete]}]) {\n        id: ID!\n        title: String!\n        createdAt: String\n        updatedAt: String\n    }\n    ";
    var transformer = new graphql_transformer_core_1.default({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({ authMode: 'AMAZON_COGNITO_USER_POOLS' })
        ]
    });
    var out = transformer.transform(validSchema);
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Query.getPost.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.getPost.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Query.listPosts.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.listPosts.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.createPost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.createPost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toMatchSnapshot();
});
test('Test that operation overwrites queries in auth operations', function () {
    var validSchema = "\n    type Post @model @auth(rules: [{allow: groups, groups: [\"Admin\", \"Dev\"], queries: [get, list], operations: [create, update, delete]}]) {\n        id: ID!\n        title: String!\n        createdAt: String\n        updatedAt: String\n    }\n    ";
    var transformer = new graphql_transformer_core_1.default({
        transformers: [
            new graphql_dynamodb_transformer_1.DynamoDBModelTransformer(),
            new ModelAuthTransformer_1.ModelAuthTransformer({ authMode: 'AMAZON_COGNITO_USER_POOLS' })
        ]
    });
    var out = transformer.transform(validSchema);
    expect(out).toBeDefined();
    expect(out.rootStack.Resources[graphql_transformer_common_1.ResourceConstants.RESOURCES.GraphQLAPILogicalID].Properties.AuthenticationType).toEqual('AMAZON_COGNITO_USER_POOLS');
    expect(out.resolvers['Query.getPost.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.getPost.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Query.listPosts.res.vtl']).not.toContain('Authorization rule:');
    expect(out.resolvers['Query.listPosts.res.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.createPost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.createPost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.updatePost.req.vtl']).toMatchSnapshot();
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toContain('Authorization rule:');
    expect(out.resolvers['Mutation.deletePost.req.vtl']).toMatchSnapshot();
});
//# sourceMappingURL=OperationsArgument.test.js.map